const std = @import("std");
const shell = @import("shell.zig");

const Allocator = std.mem.Allocator;

pub const PackedUserSize = @sizeOf(PackedUser);
pub const PackedUser = packed struct {
    uid: u32,
    gid: u32,
    additional_gids_offset: u29,
    shell_here: u1,
    shell_len_or_place: u6,
    homedir_len: u6,
    username_is_a_suffix: u1,
    username_offset_or_len: u5,
    gecos_len: u8,
};

pub const User = struct {
    uid: u32,
    gid: u32,
    name: []const u8,
    gecos: []const u8,
    home: []const u8,
    shell: []const u8,
};

// UserWriter accepts a naive User struct and returns a PackedUser
pub const UserWriter = struct {
    allocator: Allocator,

    pub fn init(allocator: Allocator) UserWriter {
        return UserWriter{
            .allocator = allocator,
        };
    }

    pub fn fromUser(user: User, shellw: shell.ShellWriter) !PackedUser {
        var shell_here: u1 = undefined;
        var shell_len_or_place: u6 = undefined;
        if (shellw.getIndex(user.shell)) |idx| {
            shell_here = false;
            shell_len_or_place = idx;
        } else {
            shell_here = true;
            shell_len_or_place = user.shell.len;
        }

        var puser = PackedUser{
            .uid = user.uid,
            .gid = user.gid,
            .additional_gids_offset = 0, // second pass
            .shell_here = shell_here,
            .shell_len_or_place = shell_len_or_place,
            .homedir_len = undefined,
            .username_is_a_suffix = undefined,
            .username_offset_or_len = undefined,
            .gecos_len = undefined,
        };

        _ = shellw;
        return puser;
    }
};

const testing = std.testing;

test "PackedUser is byte-aligned" {
    try testing.expectEqual(0, @rem(@bitSizeOf(PackedUser), 8));
}
